# -*- coding: utf-8 -*-

from logilab.common.testlib import TestCase, unittest_main

from pybill.lib.pdfwriters import PDFWriter
from pybill.lib.entities.accounting_docs import (Bill, PreviousAccountingDoc)

from generic_test_pdfwriter import PDFWriterGenericTest


class PDFWriterBillTest(TestCase, PDFWriterGenericTest):
    def setUp(self):
        """
        Called before each test from this class.
        """
        self.writer = PDFWriter()
        self.acc_doc = Bill("identifier")
        self.acc_doc.cfg_data = self._build_config_data()        
        self._fill_acc_doc_metadata(self.acc_doc)
        self.acc_doc.sender = self._build_person_address(u"sd")
        self.acc_doc.receiver = self._build_person_address(u"rc")
        self.acc_doc.payment_terms = u"payment_terms"
        self.entity_name = "bill"

    def test_write_three_items_and_one_downpayment(self):
        """
        Tests the PDFWriter writes correctly an accounting document with three
        items and one previously charged downpayment.
        """
        self.acc_doc.acc_items = self._build_acc_items_list(3)
        dwp = PreviousAccountingDoc()
        dwp.total = 5.0
        dwp.accdoc_id = "bill_id1"
        dwp.accdoc_date = "bill_date1"
        self.acc_doc.charged_downpayments.append(dwp)
        self._run_pdf_test("%s_three_items_and_one_downpayment.pdf"
                           % self.entity_name)

    def test_write_three_items_and_two_downpayments(self):
        """
        Tests the PDFWriter writes correctly an accounting document with three
        items and two previously charged downpayments.
        """
        self.acc_doc.acc_items = self._build_acc_items_list(3)
        dwp = PreviousAccountingDoc()
        dwp.total = 5.0
        dwp.accdoc_id = "bill_id1"
        dwp.accdoc_date = "bill_date1"
        self.acc_doc.charged_downpayments.append(dwp)
        dwp = PreviousAccountingDoc()
        dwp.total = 7.0
        dwp.vat_amount = 1.0
        dwp.accdoc_id = "bill_id2"
        dwp.accdoc_date = "bill_date2"
        self.acc_doc.charged_downpayments.append(dwp)
        self._run_pdf_test("%s_three_items_and_two_downpayments.pdf"
                           % self.entity_name)

    def test_write_three_items_and_one_debit(self):
        """
        Tests the PDFWriter writes correctly an accounting document with three
        items and one previously issued debit.
        """
        self.acc_doc.acc_items = self._build_acc_items_list(3)
        dbt = PreviousAccountingDoc()
        dbt.total = 5.0
        dbt.accdoc_id = "debit_id1"
        dbt.accdoc_date = "debit_date1"
        self.acc_doc.issued_debits.append(dbt)
        self._run_pdf_test("%s_three_items_and_one_debit.pdf"
                           % self.entity_name)

    def test_write_three_items_and_two_debits(self):
        """
        Tests the PDFWriter writes correctly an accounting document with three
        items and two previously issued debits.
        """
        self.acc_doc.acc_items = self._build_acc_items_list(3)
        dbt = PreviousAccountingDoc()
        dbt.total = 5.0
        dbt.accdoc_id = "debit_id1"
        dbt.accdoc_date = "debit_date1"
        self.acc_doc.issued_debits.append(dbt)
        dbt = PreviousAccountingDoc()
        dbt.total = 7.0
        dbt.vat_amount = 1.0
        dbt.accdoc_id = "debit_id2"
        dbt.accdoc_date = "debit_date2"
        self.acc_doc.issued_debits.append(dbt)
        self._run_pdf_test("%s_three_items_and_two_debits.pdf"
                           % self.entity_name)

    def test_write_three_items_one_downpayment_and_one_debit(self):
        """
        Tests the PDFWriter writes correctly an accounting document with three
        items, one previously charged downpayment and one previously issued
        debit.
        """
        self.acc_doc.acc_items = self._build_acc_items_list(3)
        dbt = PreviousAccountingDoc()
        dbt.total = 5.0
        dbt.vat_amount = 0.5
        dbt.accdoc_id = "debit_id1"
        dbt.accdoc_date = "debit_date1"
        self.acc_doc.issued_debits.append(dbt)
        dwp = PreviousAccountingDoc()
        dwp.total = 11.0
        dwp.vat_amount = 2.0
        dwp.accdoc_id = "bill_id1"
        dwp.accdoc_date = "bill_date1"
        self.acc_doc.charged_downpayments.append(dwp)
        self._run_pdf_test("%s_three_items_one_downpayment_and_one_debit.pdf"
                           % self.entity_name)


# definitions for automatic unit testing

if __name__ == '__main__':
    unittest_main()



# -*- coding: utf-8 -*-

from pybill.lib import ACCURACY, PBD_1_0, PBD_0_X
import datetime


class FormatReaderGenericTest:
    """
    Abstract class containing various useful methods for testing XML accounting
    documents reading.
    """
    def __init__(self):
        if self.__class__ is FormatReaderGenericTest:
            raise NotImplementedError()

    def _check_doc_generic_content(self, acc_doc, format=PBD_1_0,
                                   check_items=True, check_id=True):
        """
        Checks document generic attributes data (i.e. attributes common to all
        accounting documents
        
        :parameter acc_doc: Object whose content will be tested. This
                            object has been produced by an XML reader.
        :type acc_doc: GenericAccountingDoc
        
        :parameter format: Format of the data that has been read. The format
                           leads to different reading procedures (default
                           values, etc.) and thus different pieces of data. Can
                           be ``u"PBD-1.0"`` (default value) or ``u"PBD-0.X"``.
        :type format: unicode

        :parameter check_items: When set to True (default value), this method
                                will check the items in the ``acc_items`` list
                                of the accounting document object (``acc_doc``).
        :type check_items: bool

        :parameter check_id: When set to True (default value), this method
                             will check the value of the id of the accounting
                             document object (``acc_doc``).
        :type check_id: bool
        """
        # Checks document metadata
        if check_id:
            self.assertEqual(acc_doc.id, u"A1")
        self.assertEqual(acc_doc.origin_format, format)
        self.assertEqual(acc_doc.doc_ref, u"A2")
        self.assertEqual(acc_doc.place, u"A3")
        self.assertEqual(acc_doc.date, u"A4")
        if format == PBD_1_0:
            self.assertEqual(acc_doc.date_num, datetime.date(2009, 10, 19))
        self.assertEqual(acc_doc.other_infos, {u"A5": u"A6", u"A7": u"A8",
                                                u"A9": u"A10"})
        # Checks the addresses
        self._check_acc_doc_address(acc_doc.sender, u'B')
        self._check_acc_doc_address(acc_doc.receiver, u'C')
        # Checks remarks
        self.assertEqual(acc_doc.remarks, [u"D1", u"D2"])
        # Checks accounting items
        if check_items:
            self.assertEqual(len(acc_doc.acc_items), 4)
            if format == PBD_1_0:
                expected_cnt = [ (501.0, 5, u"E2", [u"E3", u"E4"], 505.0,
                                  ACCURACY, None, None, False),
                                 (601.0, 0, u"F2", [], 603.0, 6, 604.0,
                                  None, False),
                                 (701.0, 0, u"G2", [], 703.0, ACCURACY, 704.0,
                                  7.0, False),
                                 (801.0, 0, u"H2", [], 803.0, ACCURACY, 804.0,
                                  8.0, True),
                               ]
            elif format == PBD_0_X:
                expected_cnt = [ (501.0, 1, u"E2", [u"E3",u"E4"], 505.0,
                                  ACCURACY, 19.6, None, False),
                                 (601.0, 1, u"F2", [], 603.0, ACCURACY, 604.0,
                                  None, False),
                                 (701.0, 1, u"G2", [], 703.0, ACCURACY, 704.0,
                                  None, False),
                                 (801.0, 1, u"H2", [], 803.0, ACCURACY, 804.0,
                                  None, False),
                               ]
            else:
                # Unknown format
                expected_cnt = [ (None, None, None, None, None, None, None,
                                  None, None) ]*4
            for i in range(4):
                self._check_sold_item_with_content(acc_doc.acc_items[i],
                                                   expected_cnt[i])
        # Checks the config object (that is still not defined)
        self.assertEqual(acc_doc.cfg_data, None)

    def _check_acc_doc_address(self, addr, code):
        """
        Checks the address of the accounting document contains expected data

        :parameter code: either u"B" or u"C" (first letter of the address data
                         content)
        :parameter code: unicode
        """
        self.assertEqual(addr.honorific, u"%s1" % code)
        self.assertEqual(addr.firstname, u"%s2" % code)
        self.assertEqual(addr.other_name, u"%s3" % code)
        self.assertEqual(addr.surname, u"%s4" % code)
        self.assertEqual(addr.lineage, u"%s5" % code)
        self.assertEqual(addr.streets,[u"%s6" % code, u"%s7" % code])
        self.assertEqual(addr.post_box, u"%s8" % code)
        self.assertEqual(addr.post_code, u"%s9" % code)
        self.assertEqual(addr.city, u"%s10" % code)
        self.assertEqual(addr.state, u"%s11" % code)
        self.assertEqual(addr.country, u"%s12" % code)
        self.assertEqual(addr.phone, u"%s13" % code)
        self.assertEqual(addr.fax, u"%s14" % code)
        self.assertEqual(addr.web, u"%s15" % code)
        self.assertEqual(addr.email, u"%s16" % code)
        # affiliation
        self.assertEqual(addr.organisation.name, u"%s19" % code)
        self.assertEqual(addr.organisation.job_titles, [u"%s17" % code,
                                                         u"%s18" % code])
        self.assertEqual(addr.organisation.divisions, [u"%s20" % code,
                                                        u"%s21" % code])
        self.assertEqual(addr.organisation.streets, [u"%s22" % code,
                                                      u"%s23" % code])
        self.assertEqual(addr.organisation.post_box, u"%s24" % code)
        self.assertEqual(addr.organisation.post_code, u"%s25" % code)
        self.assertEqual(addr.organisation.city, u"%s26" % code)
        self.assertEqual(addr.organisation.state, u"%s27" % code)
        self.assertEqual(addr.organisation.country, u"%s28" % code)
        self.assertEqual(addr.organisation.phone, u"%s29" % code)
        self.assertEqual(addr.organisation.fax, u"%s30" % code)
        self.assertEqual(addr.organisation.web, u"%s31" % code)
        self.assertEqual(addr.organisation.email, u"%s32" % code)

    def _check_sold_item_with_content(self, acc_it, content):
        """
        Compares the AccItem 'acc_it' with the data contained in the tuple
        'content'.

        :parameter acc_it: Accounting item whose content is to be compared with
                           the expected content.
        :type acc_it: AccItem

        :parameter content: tuple containing the expected values for the
                            accounting object content. These values are:
                            (quantity, number of digits for the quantity,
                            title, list of details, unit price, number of
                            digits for the unit price, VAT rate,
                            holdback rate, boolean set when the holdback applies
                            on the VAT)
        :type content: (float, int, unicode, list, float, float, float, bool)
        """
        self.assertEqual(acc_it.quantity, content[0])
        self.assertEqual(acc_it.quantity_digits, content[1])
        self.assertEqual(acc_it.title, content[2])
        self.assertEqual(acc_it.details, content[3])
        self.assertEqual(acc_it.unit_price, content[4])
        self.assertEqual(acc_it.unit_price_digits, content[5])
        self.assertEqual(acc_it.vat_rate, content[6])
        self.assertEqual(acc_it.holdback_rate, content[7])
        self.assertEqual(acc_it.holdback_on_vat, content[8])



# -*- coding: utf-8 -*-
"""
``pybill.lib.entities.addresses`` module contains the definition of the entities
related to the address description.

These entities contain data information related to person or company addresses,
and are used by other classes.

.. autoclass:: Address
   :members:
   :undoc-members:
   :show-inheritance: 

.. autoclass:: PersonAddress
   :members:
   :undoc-members:
   :show-inheritance: 

.. autoclass:: OrganisationAddress
   :members:
   :undoc-members:
   :show-inheritance: 

"""
__docformat__ = "restructuredtext en"


class Address:
    """
    Abstract class containing common parts for ``PersonAddress`` and
    ``OrganisationAddress`` classes.

    .. attribute:: streets

       Attribute containing the several `street` lines of the address (because
       the `street` definition can be complex and need several lines). Can be
       empty.

       type: list of :class:`unicode`

    .. attribute:: post_box

       Attribute defining the post box described in the address. Can be empty.
       
       type: :class:`unicode`

    .. attribute:: post_code

       Attribute defining the post code of the address. Can be empty but is
       often filled.

       type: :class:`unicode`

    .. attribute:: city

       Attribute defining the city described in the address. Can be empty but is
       often filled.

       type: :class:`unicode`

    .. attribute:: state

        Attribute containing the state described in the address. In some
        countries, the addresses don't contain a specification of state. Can be
        empty.

        type: :class:`unicode`

    .. attribute:: country

        Attribute defining the country of the address. Is often not filled when
        the sender and the receiver are in the same country. Can be empty.

        type: :class:`unicode`

    .. attribute:: phone

       Attribute containing the phone number. Only one phone number is
       allowed. Can be empty.

       type: :class:`unicode`

    .. attribute:: fax

       Attribute containing the fax number. Only one fax number is allowed. Can
       be empty.
        
       type: :class:`unicode`

    .. attribute:: web

        Attribute containing the web site address. Only one web site address is
        allowed. Can be empty.

        type: :class:`unicode`

    .. attribute:: email

       Attribute containing the email address. Only one email address is
       allowed. Can be empty.

       type: :class:`unicode`

    .. automethod:: __init__
    """
    def __init__(self):
        """
        Initializes an object.

        This object is empty and will be filled later by the XML readers.
        """
        if self.__class__ is Address:
            raise NotImplementedError("Abstract class cann't be instanciated.")
        self.streets = []
        self.post_box = u""
        self.post_code = u""
        self.city = u""
        self.state = u""
        self.country = u""
        self.phone = u""
        self.fax = u""
        self.web = u""
        self.email = u""

    def get_postal_address_lines(self):
        """
        Gives the list of the various lines of the postal address.

        Some pieces of information are displayed on the same line (e.g. post
        code and city). Some pieces of information (e.g. phone, email, etc.) are
        not in the list because they don't concern the postal address. Doesn't
        return empty lines.

        :returns: The list of the various lines of the postal address. Each line
                  is an Unicode string.
        :rtype: list of :class:`unicode`
        """
        lines = []

        for street in self.streets:
            if u" ".join(street.split()) != u'':
                lines.append(u" ".join(street.split()))

        if u" ".join(self.post_box.split()) != u'':
            lines.append(u" ".join(self.post_box.split()))

        city_line = u"%s %s %s" % (self.post_code, self.city, self.state)
        if u" ".join(city_line.split()) != u'':
            lines.append(u" ".join(city_line.split()))

        if u" ".join(self.country.split()) != u'':
            lines.append(u" ".join(self.country.split()))

        return lines


class PersonAddress(Address):
    """
    Entity class describing the address of a person (used for sender and
    receiver of the accounting documents).

    This class specializes the base class ``Address`` and contains only the
    attributes and methods to deal with the person name (other fields of the
    address are managed by the base class).

    .. attribute:: honorific

       Attribute containing the honorific used to designate the person (e.g.
       u"Mr", u"Ms", u"Dr"). Can be empty.

       type: :class:`unicode`

    .. attribute:: firstname

       Attribute containing the first name of the person. Can be empty.

       type: :class:`unicode`

    .. attribute:: other_name

       Attribute defining the other name. Other name contains other pieces of
       data concerning the name that are not a honorific, a first name, a
       surname or a lineage. These pieces of data are often written between the
       first name and the name. Can be empty.  Sometimes ``other_name`` contains
       all the name and the other fields are empty (data is not splitted accross
       all the fields).

       type: :class:`unicode`

    .. attribute:: surname

       Attribute defining the surname of the person. Can be empty.

       type: :class:`unicode`

    .. attribute:: lineage

       Attribute defining the lineage of the person (postfixes the name). E.g.
       u"Sr", u"Jr", u"3rd". Can be empty.

       type: :class:`unicode`

    .. attribute:: organisation

        Attribute containing the organisation (company, society, etc.) where the
        person works. This organisation is described with an
        ``OrganisationAddress`` object.
                        
        type: :class:`~pybill.lib.entities.addresses.OrganisationAddress`

    .. automethod:: __init__
    """
    def __init__(self):
        """
        Initializes an empty object.

        This method calls the ``__init__`` method of the base class
        (``Address``).
        """
        Address.__init__(self)
        self.honorific = u""
        self.firstname = u""
        self.other_name = u""
        self.surname = u""
        self.lineage = u""
        self.organisation = None # OrganisationAddress

    def get_person_name(self):
        """
        Returns the person name in one string (single line).

        This string is composed with the various fields concerning the name:
        ``honorific``, ``firstname``, ``other_name``, ``surname``, ``lineage``.
        
        :returns: Single line containing the name of the person.
        :rtype: :class:`unicode`
        """
        name = "%s %s %s %s %s" % (self.honorific, self.firstname,
                                   self.other_name, self.surname, self.lineage)
        return " ".join(name.split())


class OrganisationAddress(Address):
    """
    Entity class describing the address of an organisation (used in PersonAdress
    to contain data about the company of the sender/receiver).

    This class specializes the base class ``Address`` and contains only the
    attributes and methods to deal with the organisation name and the person
    role in this organisation (other fields of the address are managed by the
    base class).

    .. attribute:: name

       Attribute defining the name of the organisation. Can be empty.

       type: :class:`unicode`

    .. attribute:: divisions

       Attribute containing the list of the various divisions that the person
       working for this organisation is affiliated to. This attribute is used
       when the organisation address is embedded in a person address.  Can be
       empty.

       type: list of :class:`unicode`

    .. attribute:: job_titles

       Attribute containing a list of the various job titles that the person
       working for this organisation has got. This attribute is used when the
       organisation address is embedded in a person address. Can be empty.

       type: list of :class:`unicode`

    .. automethod:: __init__
    """
    def __init__(self):
        """
        Initializes an empty object.

        This method calls the ``__init__`` method of the base class
        (``Address``).
        """
        Address.__init__(self)
        self.name = u""
        self.divisions = []
        self.job_titles = []



# -*- coding: utf-8 -*-
"""
``pybill.lib.accounting.generator`` module defines the class that can generate
accounting entries from the accounting documents (PyBill entities).

This class can be used to generate the accounting entries for several documents
and then to save the entries into an XML file.

.. autoclass:: EntriesGenerator
   :members:
   :undoc-members:
   :show-inheritance:
"""
__docformat__ = "restructuredtext en"

import datetime
from lxml import etree

from pybill.lib import ACCURACY
from pybill.lib.xmlwriters.utils import ENCODING, open_xml_outfile
from pybill.lib.accounting.utils import CLIENT, CLIENT_HOLDBACK, PRODUCT, VAT
from pybill.lib.entities.accounting_docs import (Bill, ClaimForm, Debit,
                                                 Downpayment, ProForma)
from pybill.lib.accounting.utils import Entry


# Dictionary associating accounting document classes with keywords.
TYPENAME = {Bill: u"bill", ClaimForm: u"claim-form", Debit: u"debit",
            Downpayment: u"downpayment", ProForma: u"pro-forma"}

# Default account numbers used when no account number is specified in the
# accounting documents.
DEFAULT_PRODUCT = u"70"
DEFAULT_CLIENT = u"411"
DEFAULT_VAT = u"445"


class EntriesGenerator:
    """
    Class that generates accounting entries from the accounting documents
    (PyBill entities).

    This class generates a unique list of accounting entries from a set of
    accounting documents, each document generating a single entry. This list
    can be written to an XML file with the dedicated method.

    The accounting entries are generated in `pycompta` format.

    .. attribute:: entries

        Attribute containing a list of accounting entries generated from the
        accounting documents. When generating the entry for a new accounting
        document, it will be appended to this list.

        This list is usually emptied after the entries have been written into an
        XML document.

        type: list of :class:`~pybill.lib.accounting.utils.Entry`

    .. automethod:: __init__
    """
    def __init__(self):
        """
        Initializes the generator.
        """
        self.entries = None
        self.reset_entries()

    def reset_entries(self):
        """
        Erases all the previously generated entries and starts with a new
        empty list of entries.
        """
        self.entries = []

    def generate_entry(self, acc_doc):
        """
        Generates the accounting entry from the ``acc_doc`` document and adds
        it to the list in this class.

        :parameter acc_doc: Accounting document from which the accounting entry
                            will be generated.
        :type acc_doc:
            :class:`~pybill.lib.entities.accounting_docs.abstract.GenericAccountingDoc`
        """
        if acc_doc.__class__ in [ProForma]:
            # Nothing to do as ``acc_doc`` is only a hypothetic future bill.
            pass
        else:
            # Builds a new accounting entry with the date & explanation fields.
            if acc_doc.date_num is not None:
                acc_date = acc_doc.date_num
            else:
                acc_date = datetime.date.today()
            expl = (u"%s %s %s" % (
                acc_doc.cfg_data.get_local_phrase(TYPENAME[acc_doc.__class__]),
                acc_doc.cfg_data.get_local_phrase(u"number"),
                acc_doc.id))
            entry = Entry(acc_date, expl)
            # Fills the accounting entry with credits and debits.
            prod_account = acc_doc.account_numbers.get(PRODUCT, DEFAULT_PRODUCT)
            vat_account = acc_doc.account_numbers.get(VAT, DEFAULT_VAT)
            cli_account = acc_doc.account_numbers.get(CLIENT, DEFAULT_CLIENT)
            hlb_account = acc_doc.account_numbers.get(CLIENT_HOLDBACK,
                                                      cli_account)
            no_hlb = acc_doc.get_to_be_paid_holdback_free()
            hlb = acc_doc.get_to_be_paid_holdback()
            vat = acc_doc.get_to_be_paid_vat()
            if acc_doc.__class__ in [Bill, ClaimForm, Downpayment]:    
                entry.add_movement(prod_account, (no_hlb + hlb - vat))
                if abs(vat) > 10**-(ACCURACY + 3):
                    entry.add_movement(vat_account, vat)
                entry.add_movement(cli_account, -no_hlb)
                if abs(hlb) > 10**-(ACCURACY + 3):
                    entry.add_movement(hlb_account, -hlb)
            elif acc_doc.__class__ in [Debit]:
                entry.add_movement(prod_account, -(no_hlb + hlb - vat))
                if abs(vat) > 10**-(ACCURACY + 3):
                    entry.add_movement(vat_account, -vat)
                entry.add_movement(cli_account, no_hlb)
                if abs(hlb) > 10**-(ACCURACY + 3):
                    entry.add_movement(hlb_account, hlb)
            else:
                raise Exception("Unknown class of accounting document for %s:"
                                " %s" % (acc_doc.id, acc_doc.__class__))
            # Checks the accounting entry and adds it to the list.
            entry.check()
            self.entries.append(entry)

    def write_entries_in_xml(self, xml_file, encoding=ENCODING):
        """
        Exports the generated accounting entries in an XML file.

        :parameter xml_file: Name of the file where the accounting entries
                             will be exported to.
        :type filename: :class:`unicode`

        :parameter encoding: Encoding used to write the XML file.
        :type encoding: :class:`str`
        """
        # Builds the XML document containing the accounting entries.
        root_xml = etree.Element(u"ecritures")
        for entry in self.entries:
            entry_xml = etree.SubElement(root_xml, u"ecriture")
            entry_xml.set(u"date", unicode(entry.date.strftime("%Y-%m-%d")))
            explan_xml = etree.SubElement(entry_xml, u"libelle")
            explan_xml.text = unicode(entry.explanation)
            for account in sorted(entry.debits):
                for amount in entry.debits[account]:
                    dbt_xml = etree.SubElement(entry_xml, u"debit")
                    dbt_xml.set(u"compte", unicode(account))
                    dbt_xml.set(u"montant", unicode(amount))
            for account in sorted(entry.credits):
                for amount in entry.credits[account]:
                    dbt_xml = etree.SubElement(entry_xml, u"credit")
                    dbt_xml.set(u"compte", unicode(account))
                    dbt_xml.set(u"montant", unicode(amount))
        # Opens the XML file and writes the XML document in it.
        if len(self.entries) > 0:
            out = open_xml_outfile(xml_file, encoding=encoding)
            out.write(etree.tostring(root_xml, encoding=encoding,
                                     pretty_print=True, xml_declaration=False))



# -*- coding: utf-8 -*-
# Copyright (c) 2001-2011 LOGILAB S.A. (Paris, FRANCE). All rights Reserved.
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
PyBill packaging information
"""
from os import path as osp


# package name
modname = 'pybill'
debian_name = 'pybill'

# release version
numversion = (1, 1, 0)
version = '.'.join([str(num) for num in numversion])

pyversions = ['2.4','2.5', '2.6']

# license and copyright
license = "GPL"
copyright = """Copyright (c) 2001-2011 LOGILAB S.A. (Paris, FRANCE). All rights Reserved.
http://www.logilab.fr/ -- mailto:contact@logilab.fr"""

# short and long description
short_desc = "PDF formatting tool for bills"
try:
    long_desc = open(osp.join(osp.abspath(osp.dirname(__file__)), "README")).read()
except Exception, exc:
    long_desc = short_desc

# author name and email
author = "Olivier Cayrol"
author_email = "olivier.cayrol@logilab.fr"

# home page
web = "http://www.logilab.org/project/pybill"

# mailing list
mailinglist = 'mail-to://management-projects@lists.logilab.org' 

# download place
ftp = "ftp://ftp.logilab.org/pub/%s" % modname

# executable (include the 'bin' directory in the name)
scripts = ('bin/pybill',)

# should it be placed as a subpackage of a package such as 'logilab'
subpackage_of = None

# is there some directories to include with the source installation
include_dirs = []



# -*- coding: utf-8 -*-
"""
``pybill.lib.entities.accounting_docs.debit`` module defines the entity
representing a debit.

.. autoclass:: Debit
   :members:
   :undoc-members:
   :show-inheritance:
"""
__docformat__ = "restructuredtext en"

from pybill.lib.entities.accounting_docs.abstract import GenericAccountingDoc
from pybill.lib import ACCURACY


class Debit(GenericAccountingDoc):
    """
    Entity class representing a debit.

    Please note that, for this class, the `to_be_paid_*` methods give the
    amounts that must be paid back to the client.

    .. automethod:: __init__
    """
    def __init__(self, debit_id):
        """
        Initializes an empty object.

        :parameter debit_id: Identifier of this accounting document.
        :type debit_id: :class:`unicode`
        """
        GenericAccountingDoc.__init__(self)
        # Filling the object with available data
        self.id = debit_id

    def get_to_be_paid_holdback_free(self):
        """
        Computes the total without holdback that must be paid back to the
        client.

        This total depends, of course, on the including taxes total but also on
        the holdbacks.
        
        :returns: The total without holdback of the debit that must be
                  paid back to the client.
        :rtype: :class:`float`
        """
        return round(self.get_including_taxes_total()
                     - self.get_holdback_amount()
                     - self.get_holdback_vat_amount(), ACCURACY)

    def get_to_be_paid_holdback(self):
        """
        Computes the total of holdbacks that must be paid back to the client.

        This total depends, of course, on the including taxes total but also
        on the holdbacks.
        
        :returns: The total of holdbacks of the debit that must be paid back to
                  the client.
        :rtype: :class:`float`
        """
        return round(self.get_holdback_amount()
                     + self.get_holdback_vat_amount(), ACCURACY)

    def get_to_be_paid_vat(self):
        """
        Computes the total of VAT amounts that must be paid back to the client.

        This amount depends on the VAT amounts.

        :returns: The total of VAT amounts that must be paid back to the
                  client.
        :rtype: :class:`float`
        """
        vat_amount = self.get_including_taxes_total() - self.get_price_total()
        return round(vat_amount, ACCURACY)



# -*- coding: utf-8 -*-
"""
``pybill.lib`` package is the library package of `PyBill`.

It contains all the core code of PyBill for reading and writing XML accounting
documents, computing each type of accounting document, writing PDF
representations of the accounting documents, and writing accounting entries.

The ``pybill.lib`` package directly contains the definitions of some constants
used all over `PyBill` code.

.. autodata:: ACCURACY

.. autodata:: PBD_1_0

.. autodata:: PBD_0_X

.. autodata:: PBD_FORMATS

.. autodata:: PBD_XMLNS

.. autodata:: PBC_1_0

.. autodata:: PBC_0_X

.. autodata:: PBC_FORMATS

.. autodata:: PBC_XMLNS
"""
__docformat__ = "restructuredtext en"

ACCURACY = 2
"""
Constant defining the accuracy of all the computations that will be done in
PyBill.

When computing the total of an item, it will be rounded at this accuracy. When
computing the total of an accounting document, the rounded total of each item
will be used and the result will be rounded at this accuracy.

type: :class:`int`

value: ``2``
"""

PBD_1_0 = u"PBD-1.0"
"""
Constant representing the format `PyBill Document 1.0`.

This constant is used for the format specification in the XML document files.

type: :class:`unicode`

value: ``PBD-1.0``
"""

PBD_0_X = u"PBD-0.X"
"""
Constant representing the older format `PyBill Document 0.X`.

This constant is used for the format specification in the XML document files.

type: :class:`unicode`

value: ``PBD-0.X``
"""

PBD_FORMATS = [PBD_1_0, PBD_0_X]
"""
Constant containing the list of the possible formats for the documents
(using the constants previously defined).

type: list of :class:`unicode`

definition: ``[PBD_1_0, PBD_0_X]``
"""

PBD_XMLNS = u"http://www.logilab.org/2010/PyBillDocument"
"""
Constant defining the namespace used for PyBill XML document in
`PyBill Document 1.0` format or higher.

type: :class:`unicode`

value: ``http://www.logilab.org/2010/PyBillDocument``
"""

PBC_1_0 = u"PBC-1.0"
"""
Constant representing the format `PyBill Configuration 1.0`.

This constant is used for the format specification in the XML configuration
files.

type: :class:`unicode`

value: ``PBC-1.0``
"""

PBC_0_X = u"PBC-0.X"
"""
Constant representing the older format `PyBill Configuration 0.X`.

This constant is used for the format specification in the XML configuration
files.

type: :class:`unicode`

value: ``PBC-0.X``
"""

PBC_FORMATS = [PBC_1_0, PBC_0_X]
"""
Constant containing the list of the possible formats for the configuration
(using the constants previously defined).

type: list of :class:`unicode`

definition: ``[PBC_1_0, PBC_0_X]``
"""

PBC_XMLNS = u"http://www.logilab.org/2010/PyBillConfig"
"""
Constant defining the namespace used for PyBill XML configuration in
`PyBill Config 1.0` format or higher.

type: :class:`unicode`

value: ``http://www.logilab.org/2010/PyBillConfig``
"""



# -*- coding: utf-8 -*-
"""
``pybill.lib.entities.accounting_docs.proforma`` module defines the entity
representing a pro-forma.

.. autoclass:: ProForma
   :members:
   :undoc-members:
   :show-inheritance:
"""
__docformat__ = "restructuredtext en"

from pybill.lib.entities.accounting_docs.abstract import GenericAccountingDoc


class ProForma(GenericAccountingDoc):
    """
    Entity class representing a pro-forma bill used as a purchase order.

    .. attribute:: validity_date

       Attribute defining the date that the purchase order is valid until; this
       is a **string representation** of the date.

       type: :class:`unicode`

    .. automethod:: __init__
    """
    def __init__(self, prof_id=None):
        """
        Initializes an empty object.

        Please note that the objects of this class don't need an identifier. The
        identifier is kept in the arguments to have the same interface as the
        others accounting entities.

        :parameter prof_id: Just kept to have the same interface as the other
                            accounting entities. This identifier is not used as
                            ``ProForma`` objects don't have identifier.
        :type prof_id: :class:`unicode`
        """
        GenericAccountingDoc.__init__(self)        
        self.validity_date = ""
        # Filling the object with available data
        self.id = None

    def get_to_be_paid_holdback_free(self):
        """
        In a pro-forma, nothing is to be paid (it's not really a bill).

        This method always returns ``0.0``\ .
        
        :returns: ``0.0`` as nothing is to be paid.
        :rtype: :class:`float`
        """
        return 0.0

    def get_to_be_paid_holdback(self):
        """
        In a pro-forma, nothing is to be paid (it's not really a bill).

        This method always returns ``0.0``\ .
        
        :returns: ``0.0`` as nothing is to be paid.
        :rtype: :class:`float`
        """
        return 0.0

    def get_to_be_paid_vat(self):
        """
        In a pro-forma, nothing is to be paid (it's not really a bill).

        This method always returns ``0.0``\ .
        
        :returns: ``0.0`` as nothing is to be paid.
        :rtype: :class:`float`
        """
        return 0.0
